﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirBomber
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetAirplanesGeneric<T>
        where T : class, IEquatable<T>
    {
        /// <summary>
        /// Список объектов, которые храним
        /// </summary>
        private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Count;

        private readonly int _maxcount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetAirplanesGeneric(int count)
        {
            _maxcount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="airplane">Добавляемый самолет</param>
        /// <returns>Возвращает позицию вставленного объекта, либо -1 если его не удалось вставить</returns>
        public int Insert(T airplane)
        {
            return Insert(airplane, 0);
        }

        private bool isCorrectPosition(int position)
        {
            return 0 <= position && position < _maxcount;
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="airplane">Добавляемый самолет</param>
        /// <param name="position">Позиция</param>
        /// <exception cref="ArgumentException"> Исключения генерируется при попытке добавить дубликат в набор</exception> 
        /// <exception cref="StorageOverflowException"> </exception>
        /// <returns>Возвращает позицию вставленного объекта, либо -1 если его не удалось вставить</returns>
        public int Insert(T airplane, int position)
        {
            if (_places.Contains(airplane))
                throw new ArgumentException($"Объект {airplane} уже есть в наборе");
            if (Count == _maxcount)
                throw new StorageOverflowException(_maxcount);
            if (!isCorrectPosition(position))
            {
                return -1;
            }
            _places.Insert(position, airplane);
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns>Возвращает удаленный объект, либо null если его не удалось удалить</returns>
        public T Remove(int position)
        {
            if (!isCorrectPosition(position))
                return null;
            var result = this[position];
            if (result == null)
                throw new AirplaneNotFoundException(position);
            _places.RemoveAt(position);
            return result;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            get
            {
                return isCorrectPosition(position) && position < Count ? _places[position] : null;
            }
            set
            {
                Insert(value, position);
            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetAirplanes()
        {
            foreach (var airplane in _places)
            {
                if (airplane != null)
                {
                    yield return airplane;
                }
                else
                {
                    yield break;
                }
            }

        }
        /// <summary>
        /// Сортировка набора объектов
        /// </summary>
        /// <param name="comparer"></param>
        public void SortSet(IComparer<T> comparer)
        {
            if (comparer == null)
            {
                return;
            }
            _places.Sort(comparer);
        }
    }
}
